<?php

/**
 * Date controller.
 *
 * @category   apps
 * @package    date
 * @subpackage controllers
 * @author     ClearFoundation <developer@clearfoundation.com>
 * @copyright  2011 ClearFoundation
 * @license    http://www.gnu.org/copyleft/gpl.html GNU General Public License version 3 or later
 * @link       http://www.clearfoundation.com/docs/developer/apps/date/
 */

///////////////////////////////////////////////////////////////////////////////
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Date controller.
 *
 * @category   apps
 * @package    date
 * @subpackage controllers
 * @author     ClearFoundation <developer@clearfoundation.com>
 * @copyright  2011 ClearFoundation
 * @license    http://www.gnu.org/copyleft/gpl.html GNU General Public License version 3 or later
 * @link       http://www.clearfoundation.com/docs/developer/apps/date/
 */

class Date extends ClearOS_Controller
{
    /**
     * Date default controller.
     *
     * @return view
     */

    function index()
    {
        $this->view();
    }

    /**
     * Date settings edit view.
     *
     * @return view
     */

    function edit()
    {
        $this->_view_edit('edit');
    }

    /**
     * Date settings view view.
     *
     * @return view
     */

    function view()
    {
        $this->_view_edit('view');
    }

    /**
     * Date default controller
     *
     * @param string $form_type form type
     *
     * @return view
     */

    function _view_edit($form_type)
    {
        // Load dependencies
        //------------------

        $this->load->library('date/Time');
        $this->load->library('date/NTP_Time');
        $this->lang->load('date');

        // Set validation rules
        //---------------------
         
        $this->form_validation->set_policy('time_zone', 'date/Time', 'validate_time_zone', TRUE);
        $this->form_validation->set_policy('auto_synchronize', 'date/NTP_Time', 'validate_state', TRUE);
        $form_ok = $this->form_validation->run();

        // Handle form submit
        //-------------------

        if (($this->input->post('time_zone') && $form_ok)) {
            try {
                $this->time->set_time_zone($this->input->post('time_zone'));
                $this->ntp_time->set_schedule_state($this->input->post('auto_synchronize'));

                $this->page->set_status_updated();

                if (empty($this->session->userdata['wizard_redirect']))
                    redirect('/date');
                else
                    redirect($this->session->userdata['wizard_redirect']);
            } catch (Exception $e) {
                $this->page->view_exception($e);
                return;
            }
        }

        // Load view data
        //---------------

        try {
            if (isset($this->session->userdata['wizard']))
                $data['is_wizard'] = TRUE;

            $data['form_type'] = $form_type;

            $data['time_zone'] = $this->time->get_time_zone();
            $data['time_zones'] = $this->time->get_time_zone_list();
            $data['auto_synchronize'] = $this->ntp_time->get_schedule_state();

            $timestamp = $this->time->get_time();
            $data['date'] = strftime("%b %e %Y", $timestamp);
            $data['time'] = strftime("%T %Z", $timestamp);
        } catch (Exception $e) {
            $this->page->view_exception($e);
            return;
        }

        // Load views
        //-----------

        $this->page->view_form('date/date', $data, lang('date_app_name'));
    }

    /**
     * Runs a network time synchronization event.
     *
     * @return string offset time
     */

    function sync()
    {
        // Load dependencies
        //------------------

        $this->load->library('date/NTP_Time');

        // Run synchronize
        //----------------

        try {
            $data['error_code'] = 0;
            $data['diff'] = $this->ntp_time->synchronize();
            $data['date'] = strftime("%b %e %Y");
            $data['time'] = strftime("%T %Z");
        } catch (Exception $e) {
            $data['error_code'] = clearos_exception_code($e);
            $data['error_message'] = clearos_exception_message($e);
        }

        // Return status message
        //----------------------

        $this->output->set_header("Content-Type: application/json");
        $this->output->set_output(json_encode($data));
    }
}  
